﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Media;
using System.Reflection;
using System.IO;
using System.Diagnostics;
using System.Windows.Controls;
using System.Security;
using System.Net;
using System.Timers;

using Ecng.Xaml;
using Ecng.Common;

using StockSharp.Algo;
using StockSharp.Algo.Storages;
using StockSharp.Algo.Candles;
using StockSharp.Algo.Indicators;
using StockSharp.Algo.Testing;
using StockSharp.BusinessEntities;
using StockSharp.Messages;
using StockSharp.Logging;

using StockSharp.Algo.Strategies;

namespace TestingCandles
{
    public partial class MainWindow : Window
    {
        public static HistoryEmulationConnector ht_connector;
        int processed_securities_count = 0; //start history emulation connector when 2 securities found in Ht_connector_NewSecurity() event
        public static Portfolio portfolio;
        public List<ASecurity> securities;
        public static List<SomeStrategy> strategies = new List<SomeStrategy>();
        public static LogManager log_manager;

        //Extended security class with StockSharp security
        public class ASecurity
        {
            private InMemoryExchangeInfoProvider exchange_info_provider = new InMemoryExchangeInfoProvider();

            //StockSharp security
            public Security security { get; set; }

            //Candle serie
            public CandleSeries serie { get; set; }

            //Candles from database
            private List<Candle> candles_source = new List<Candle>()
            {
                {new TimeFrameCandle() { OpenPrice = 2, ClosePrice = 3, HighPrice = 4, LowPrice = 1, TotalVolume = 5, OpenTime = new DateTimeOffset(2018, 5, 7, 10, 0, 0, TimeSpan.FromHours(3)), TimeFrame = TimeSpan.FromDays(1), State = CandleStates.Finished } },
                {new TimeFrameCandle() { OpenPrice = 3, ClosePrice = 3, HighPrice = 5, LowPrice = 2, TotalVolume = 4, OpenTime = new DateTimeOffset(2018, 5, 8, 10, 0, 0, TimeSpan.FromHours(3)), TimeFrame = TimeSpan.FromDays(1), State = CandleStates.Finished } },
                {new TimeFrameCandle() { OpenPrice = 3, ClosePrice = 2, HighPrice = 5, LowPrice = 2, TotalVolume = 6, OpenTime = new DateTimeOffset(2018, 5, 9, 10, 0, 0, TimeSpan.FromHours(3)), TimeFrame = TimeSpan.FromDays(1), State = CandleStates.Finished } },
            };

            public ASecurity(string _code)
            {
                security = new Security();
                security.Code = _code;
                security.Id = security.Name = _code + "@TQBR";
                var identety = security.Name.ToSecurityId();
                security.Board = exchange_info_provider.GetOrCreateBoard(identety.BoardCode);
            }

            //For subscription
            public IEnumerable<Message> getCandles(DateTimeOffset d)
            {
                return candles_source.Where(
                    c => c.OpenTime.Date.Year == d.Date.Year && c.OpenTime.Date.Month == d.Date.Month && c.OpenTime.Date.Day == d.Date.Day
                ).Select(t => t.ToMessage());
            }
        }

        //Strategy class
        public class SomeStrategy : Strategy
        {
            private CandleManager candle_manager;
            private ASecurity sec;

            public SomeStrategy(ASecurity _sec)
            {
                sec = _sec;
                this.SetCandleManager(new CandleManager(ht_connector));
            }

            protected override void OnStarted()
            {
                //Candle manager
                candle_manager = (CandleManager)this.GetCandleManager();
                candle_manager.LogLevel = LogLevels.Debug;
                log_manager.Sources.Add(candle_manager);

                //Create serie
                sec.serie = new CandleSeries(typeof(TimeFrameCandle), sec.security, TimeSpan.FromDays(1));
                candle_manager.Start(sec.serie);

                //Event to catch candle
                candle_manager
                    .WhenCandles(sec.serie)
                    .Do((candle) => processCandle(candle))
                    .Apply(this);

                base.OnStarted();
            }

            public void processCandle(Candle candle)
            {
                //TARGET !!! :)))
                Debug.WriteLine("{0} | {1}", sec.security, candle);
            }
        }

        public MainWindow()
        {
            InitializeComponent();

            //Create log manager
            log_manager = new LogManager();
            var fileLogListener = new FileLogListener("sample.log");
            log_manager.Listeners.Add(fileLogListener);

            //Create 2 securities
            securities = new List<ASecurity>() { { new ASecurity("GASP") }, { new ASecurity("LKOH") } };

            //Create portfolio
            portfolio = new Portfolio() { BeginValue = 1000000, Name = "TestPortfolio" };

            //Create history emulation connector
            ht_connector = new HistoryEmulationConnector(securities.Select(s => s.security), new[] { portfolio })
            {
                EmulationAdapter = { Emulator = { Settings = { MatchOnTouch = true, } } },
                HistoryMessageAdapter =
                {
                    StartDate = new DateTimeOffset(2018,5,7,10,0,0, TimeSpan.FromHours(3)),
                    StopDate = new DateTimeOffset(2018,5,9,10,0,0, TimeSpan.FromHours(3)),
                    MarketTimeChangedInterval = TimeSpan.FromMinutes(15),
                },
                MarketTimeChangedInterval = TimeSpan.FromMinutes(15),
            };
            ht_connector.LogLevel = LogLevels.Debug;
            log_manager.Sources.Add(ht_connector);

            //history emulation connector events
            ht_connector.Connected += Ht_connector_Connected;
            ht_connector.NewSecurity += Ht_connector_NewSecurity;

            //Connect
            ht_connector.Connect();
        }

        private void Ht_connector_StateChanged() { }

        private void Ht_connector_NewSecurity(Security security)
        {
            //Subscribe to market data
            ht_connector.RegisterSecurity(security);
            ht_connector.RegisterTrades(security);

            //Create strategy
            var strategy = new SomeStrategy(securities.FirstOrDefault(s => s.security.Code == security.Code))
            {
                Volume = 1,
                Portfolio = portfolio,
                Security = security,
                Connector = ht_connector,
                LogLevel = LogLevels.Debug,
                Name = string.Format("strategy_{0}", security),
                DisposeOnStop = true
            };
            log_manager.Sources.Add(strategy);
            strategies.Add(strategy);

            processed_securities_count++;

            //If all securities found
            if (processed_securities_count == securities.Count)
            {
                //Start strategies
                foreach (var str in strategies)
                    str.Start();

                //Start emulation
                ht_connector.Start();
            }

        }

        private void Ht_connector_Connected()
        {
            //Subscribe to external history source foreach security
            foreach (var s in securities)
                ht_connector.RegisterHistorySource(s.security, MarketDataTypes.CandleTimeFrame, TimeSpan.FromDays(1), s.getCandles);
        }
    }
}
